import logging

import requests
from whatsmyip.ip import get_ip
from whatsmyip.providers import GoogleDnsProvider

from ..constants import apiversion
from ..helper.config import config
from . import urls
from .decorators import fallback, uses_session

_session_id = None


@fallback(-1)
def is_logged_in():
    if config.get("uid") is None:
        return -1

    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.get(urls.discord, params=body)
    logged_in = response.json()["discord_login"]
    return 1 if logged_in else 0


@fallback(False)
def login(uid, login_code):
    body = {"uid": uid, "login_code": login_code, "apiversion": apiversion}
    reponse = requests.post(urls.discord, json=body)
    result = reponse.json()

    if "new_id" in result:
        config.set("uid", result["new_id"])

    return result["success"]


@fallback(False)
def logout():
    body = {"uid": config.get("uid"), "apiversion": apiversion}
    reponse = requests.delete(urls.discord, json=body)
    result = reponse.json()
    return result["success"]


@fallback(None)
def register_user():
    ip = get_ip(GoogleDnsProvider)
    body = {"ip": ip, "apiversion": apiversion}
    response = requests.post(urls.user, json=body)
    result = response.json()
    return result["uid"]


@fallback(None)
def send_notification(message):
    if not is_subbed()[0]:
        return False

    body = {"uid": config.get("uid"), "message": message, "apiversion": apiversion}
    requests.post(urls.notify, json=body)


@uses_session
@fallback(None)
def send_fish_caught(fish_caught, hole_time, fish_times):
    hole_data = {
        "fish_caught": fish_caught,
        "hole_time": hole_time,
        "fish_times": fish_times,
        "session": get_session()
    }

    body = {"uid": config.get("uid"), "hole_data": hole_data, "apiversion": apiversion}
    requests.post(urls.hole_depleted, json=body)


@fallback(False)
def sub():
    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.post(urls.subscription, json=body)
    result = response.json()
    return result["success"]


@fallback((False, False))
def is_subbed():
    """
    :return: Tuple[is_subbed, success]
    """

    if config.get("uid") is None:
        return False, False

    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.get(urls.subscription, params=body)

    if response.status_code != 200:
        return False, False

    _is_subbed = response.json()["subbed"]
    return _is_subbed, True


@fallback(None)
def unsub():
    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.delete(urls.subscription, json=body)
    result = response.json()
    return result["success"]


def get_session(lazy=True):
    global _session_id

    # lazy loading logic
    if lazy and _session_id is not None:
        return _session_id

    # check if user has uid
    uid = config.get("uid")

    # then create session
    if uid:
        _session_id, online = _create_new_session(uid)
    # if not, create new id then try creating session again
    else:
        uid = register_user()
        config.set("uid", uid, True)
        logging.debug(f"New User, generated new uid: {uid}")
        if uid:
            _session_id, online = _create_new_session(uid)
        else:
            online = False

    # when the user is already registered but session is not created as uid is not found
    if online and not _session_id:
        logging.error("user not found, generating new uid.. contact dev if you don't want to loose data")
        new_uid = register_user()
        _session_id, online = _create_new_session(new_uid)
        config.set("uid", new_uid, True)
        config.set("old_uid", uid, True)

    return _session_id


@fallback((None, False))
def _create_new_session(uid):
    body = {"uid": uid, "apiversion": apiversion}
    response = requests.post(urls.session, params=body)

    if response.status_code == 405:
        return None, True

    return response.json()["session_id"], True


@fallback(False)
def has_beta():
    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.get(urls.beta, params=body)
    result = response.json()

    if not result["success"]:
        return False

    return response.json()["beta"]


@fallback(None)
def ping():
    body = {"uid": config.get("uid"), "apiversion": apiversion}
    response = requests.post(urls.ping, params=body)
    logging.debug(f"ping response: {response.json()}")



import logging
import time
import tkinter as tk
from multiprocessing import Process, Queue
from threading import Thread

from PIL import Image, ImageTk

from fishy.helper import helper
from fishy.helper.config import config


def show(win_loc, q):
    logging.debug("started splash process")
    dim = (300, 200)
    top = tk.Tk()

    top.overrideredirect(True)
    top.lift()
    top.attributes('-topmost', True)

    top.title("Loading...")
    top.resizable(False, False)
    top.iconbitmap(helper.manifest_file('icon.ico'))

    canvas = tk.Canvas(top, width=dim[0], height=dim[1], bg='white')
    canvas.pack()
    top.image = Image.open(helper.manifest_file('fishybot_logo.png')).resize(dim)
    top.image = ImageTk.PhotoImage(top.image)
    canvas.create_image(0, 0, anchor=tk.NW, image=top.image)

    # Position splash at the center of the main window

    default_loc = (str(top.winfo_reqwidth()) + "+" + str(top.winfo_reqheight()) + "+" + "0" + "0")
    loc = (win_loc or default_loc).split("+")[1:]
    top.geometry("{}x{}+{}+{}".format(dim[0], dim[1], int(loc[0]) + int(dim[0] / 2), int(loc[1]) + int(dim[1] / 2)))

    def waiting():
        q.get()
        time.sleep(0.2)
        running[0] = False
    Thread(target=waiting).start()

    running = [True]
    while running[0]:
        top.update()
        time.sleep(0.1)

    top.destroy()
    logging.debug("ended splash process")


def create_finish(q):
    def finish():
        q.put("stop")

    return finish


def start():
    q = Queue()
    Process(target=show, args=(config.get("win_loc"), q,)).start()
    return create_finish(q)



apiversion = 2

# removed since 0.5.3
chalutier = ("Chalutier", "https://www.esoui.com/downloads/dl2934/Chalutier_1.1.4.zip", 114)

# addons used
lam2 = ("LibAddonMenu-2.0", "https://www.esoui.com/downloads/dl7/LibAddonMenu-2.0r32.zip", 32)
fishyqr = ("FishyQR", "https://github.com/fishyboteso/FishyQR/releases/download/v1.2.0/FishyQR.zip", 120)
libgps = ("LibGPS", "https://cdn.esoui.com/downloads/file601/LibGPS_3_0_3.zip", 30)

version = "0.5.10"



import cv2
import numpy as np
from pyzbar.pyzbar import decode, ZBarSymbol


def image_pre_process(img):
    scale_percent = 100  # percent of original size
    width = int(img.shape[1] * scale_percent / 100)
    height = int(img.shape[0] * scale_percent / 100)
    dim = (width, height)
    img = cv2.resize(img, dim, interpolation=cv2.INTER_AREA)
    return img


def get_qr_location(og_img):
    """
    code from https://stackoverflow.com/a/45770227/4512396
    """
    gray = cv2.bilateralFilter(og_img, 11, 17, 17)
    kernel = np.ones((5, 5), np.uint8)
    erosion = cv2.erode(gray, kernel, iterations=2)
    kernel = np.ones((4, 4), np.uint8)
    img = cv2.dilate(erosion, kernel, iterations=2)

    cnt, h = cv2.findContours(img, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    valid_crops = []
    for i in range(len(cnt)):
        area = cv2.contourArea(cnt[i])
        if 500 < area < 100000:
            mask = np.zeros_like(img)
            cv2.drawContours(mask, cnt, i, 255, -1)
            x, y, w, h = cv2.boundingRect(cnt[i])
            qr_result = decode(og_img[y:h + y, x:w + x],
                               symbols=[ZBarSymbol.QRCODE])
            if qr_result:
                valid_crops.append(((x, y, x + w, y + h), area))

    return min(valid_crops, key=lambda c: c[1])[0] if valid_crops else None


# noinspection PyBroadException
def get_values_from_image(img):
    for qr in decode(img, symbols=[ZBarSymbol.QRCODE]):
        vals = qr.data.decode('utf-8').split(",")
        return tuple(float(v) for v in vals)
    return None
